# (C) 1992-2015 Altera Corporation. All rights reserved.                         
# Your use of Altera Corporation's design tools, logic functions and other       
# software and tools, and its AMPP partner logic functions, and any output       
# files any of the foregoing (including device programming or simulation         
# files), and any associated documentation or information are expressly subject  
# to the terms and conditions of the Altera Program License Subscription         
# Agreement, Altera MegaCore Function License Agreement, or other applicable     
# license agreement, including, without limitation, that your use is for the     
# sole purpose of programming logic devices manufactured by Altera and sold by   
# Altera or its authorized distributors.  Please refer to the applicable         
# agreement for further details.                                                 
    


package require ::quartus::flow
package require ::quartus::project

post_message "Running post-flow script"

# Dynamically determine the project name by finding the qpf in the directory.
set qpf_files [glob -nocomplain *.qpf]

if {[llength $qpf_files] == 0} {
    error "No QSF detected"
} elseif {[llength $qpf_files] > 1} {
    post_message "Warning: More than one QSF detected. Picking the first one."
}

set qpf_file [lindex $qpf_files 0]
set project_name [string range $qpf_file 0 [expr [string first . $qpf_file] - 1]]

post_message "Project name: $project_name"

project_open $project_name

# Run PLL adjust script
post_message "Running PLL adjustment script"
if {[catch {set sdk_root $::env(ALTERAOCLSDKROOT)} result]} {
    post_message -type error "OpenCL SDK installation not found.  Make sure ALTERAOCLSDKROOT is correctly set" -submsgs [list "Guaranteed timing flow not run - you may have timing failures on the kernel clock\n"]
} else { 
  source $::env(ALTERAOCLSDKROOT)/ip/board/bsp/adjust_plls.tcl
}


# Generate fpga.bin used for reprogramming
post_message "Generating fpga.bin"

###### TODO: Replace everything below with tcl code to generate fpga.bin
#
# Fpga.bin is board-specific.  To the OpenCL compiler it is a black box
# that gets passed to the MMD's reprogram subroutine and to the aocl program
# utility.  If you're using an MMD and aocl program utility from another board
# package, you must also replicate how that same board package generates 
# fpga.bin.
#######

post_message -type error "Generation of fpga.bin not implemented!" -submsgs [ list "The OpenCL compiler expects the file fpga.bin to be generated during Quartus compilation - this file is board-specific and will be passed back to the MMD layer and program utility when the user wishes to load a new FPGA design" "Please edit scripts/post_flow.tcl to generate fpga.bin" ]
exit 2
