#ifndef ACL_PCIE_CONFIG_H
#define ACL_PCIE_CONFIG_H

/* (C) 1992-2015 Altera Corporation. All rights reserved.                          */
/* Your use of Altera Corporation's design tools, logic functions and other        */
/* software and tools, and its AMPP partner logic functions, and any output        */
/* files any of the foregoing (including device programming or simulation          */
/* files), and any associated documentation or information are expressly subject   */
/* to the terms and conditions of the Altera Program License Subscription          */
/* Agreement, Altera MegaCore Function License Agreement, or other applicable      */
/* license agreement, including, without limitation, that your use is for the      */
/* sole purpose of programming logic devices manufactured by Altera and sold by    */
/* Altera or its authorized distributors.  Please refer to the applicable          */
/* agreement for further details.                                                  */
    


/* ===- acl_pcie_config.h  -------------------------------------------- C++ -*-=== */
/*                                                                                 */
/*                         Altera OpenCL MMD Driver                                */
/*                                                                                 */
/* ===-------------------------------------------------------------------------=== */
/*                                                                                 */
/* This file declares the class to handle functions that program the FPGA.         */
/* The actual implementation of the class lives in the acl_pcie_config.cpp,        */
/* so look there for full documentation.                                           */
/*                                                                                 */
/* ===-------------------------------------------------------------------------=== */


class ACL_PCIE_CONFIG
{
   public:
      ACL_PCIE_CONFIG(WDC_DEVICE_HANDLE device);
      ~ACL_PCIE_CONFIG();
      
      // Change the core only via PCIe, using an in-memory image of the core.rbf
      // This is supported only for Stratix V and newer devices. 
      // Return 0 on success.
      int program_core_with_CvP_image(int *core_bitstream, size_t core_rbf_len);

      // Program using an in-memory image of RBF (Not core rbf) file
      // This is used for ARM only
      // Return 0 on success. 
      int program_with_RBF_image(char *rbf_image, size_t rbf_len);

      // Program the FPGA using a given SOF file
      // Return 0 on success.
      int program_with_SOF_file(const char *filename);

      // Functions to save/load control registers from PCI Configuration Space
      // Return 0 on success.
      int save_pci_control_regs();
      int load_pci_control_regs();
      
      // Functions to query the PCI related information
      // Use NULL as input for the info that you don't care about
      // Return 0 on success.
      int query_pcie_info(unsigned int *pcie_gen, unsigned int *pcie_num_lanes, char *pcie_slot_info_str);

      // Control FPGA to HPS bridges (ARM only)
      static void enable_bridges();
      static void disable_bridges();
      
      static int fpga_in_user_mode();

   private:
#if defined(WINDOWS)
      // Retrain the PCIe link after programming FPGA with SOF file (Windows only)
      // Return 0 on success.
      int  retrain_link            (WD_PCI_SLOT &slot);

      // Helper functions for finding the PCIe related stuff. (Windows only)
      // Return 1 when target is found.
      int  find_upstream_slot      (WD_PCI_SLOT &slot, WD_PCI_SLOT *upstream);
      int  find_capabilities       (WD_PCI_SLOT &slot, DWORD *offset);      
      int  find_extended_capability(WD_PCI_SLOT &slot, UINT16 ex_cap_id_to_find, DWORD *offset);  

      // Helper functions for "program_core_with_CvP_image". (Windows only)
      // These functions are ported from "/linux64/driver/aclpci_cvp.c".
      void          get_mask_and_offset (unsigned char whichBit, unsigned char *bitMask, unsigned char *bitRegOffset);
      unsigned char wait_for_bit        (WDC_DEVICE_HANDLE device, unsigned char whichBit, unsigned char bitValue);      
      unsigned char read_bit            (WDC_DEVICE_HANDLE device, unsigned char whichBit);   
      void          write_bit           (WDC_DEVICE_HANDLE device, unsigned char whichBit, unsigned char bitValue);
      void          CVP_DRV_SetNumClks  (WDC_DEVICE_HANDLE device, unsigned char numClks);
      void          switch_clock        (WDC_DEVICE_HANDLE device);      
      void          prepare_for_pgm_data(WDC_DEVICE_HANDLE device);
      size_t        send_pgm_data       (WDC_DEVICE_HANDLE device, unsigned int *data, size_t numWords);

      static const unsigned int CONFIG_SPACE_SIZE = 0x1000;
      char m_config_space[CONFIG_SPACE_SIZE];
      WD_PCI_SLOT m_slot;
      WD_PCI_SLOT m_upstream;      
#endif   // WINDOWS
      
      WDC_DEVICE_HANDLE   m_device; 
};

#endif // ACL_PCIE_CONFIG_H
